######### 2.1.1 The AR(1) time series

par(mfrow=c(2,1))
phi1 <- 0.85 ; phi2 <- -0.85
set.seed(1001)
y1<-arima.sim(n=200,list(ar=phi1),sd=1.0)
plot(y1,type="l",main=bquote(Y[t]==.(phi1)*Y[t-1]+epsilon[t]))
set.seed(1001)
y2<-arima.sim(n=200,list(ar=phi2),sd=1.0)
plot(y2,type="l",main=bquote(Y[t]==.(phi2)*Y[t-1]+epsilon[t]))
par(mfrow=c(1,1))

######### 2.1.2 The AR(2) time series

ARMAtoMA(ar=c(1/6,-1/6),lag.max=8)
# [1] 0.1666666667 -0.1388888889 -0.0509259259 0.0146604938
# [5] 0.0109310700 -0.0006215706 -0.0019254401 -0.0002173116
plot(ARMAtoMA(ar=c(1/6,-1/6),lag.max=8)) # to get a graph
     


phi1 <- 0.9 ; phi2 <- -0.85 ; hmax <- 20
xx <- 1:hmax
acfar2 <- ARMAacf(ar=c(phi1,phi2),lag.max=hmax)
plot(xx,acfar2[-1],type="h",xlab="ACF",ylab="",main=bquote(Y[t]==.(phi1)*Y[t-1]~.(phi2)*Y[t-2]+epsilon[t]),lwd=2,ylim=c(-1.0,1.0))
abline(a=0,b=0)
pacfar2 <- ARMAacf(ar=c(phi1,phi2),pacf=TRUE,lag.max=hmax)
plot(xx,pacfar2,type="h",xlab="PACF",ylab="",main=bquote(Y[t]==.(phi1)*Y[t-1]~.(phi2)*Y[t-2]+epsilon[t]),lwd=2,ylim=c(-1.0,1.0))
abline(a=0,b=0)

######### Example 2

#Salesfr <- scan("Salesfile.txt")
Salesfr <- scan(file.choose())
Salests<-ts(Salesfr,start=c(2000,8),frequency=12)
LSales<-log10(Salests)
par(mfrow=c(2,1))
plot(Salests,lwd=2) # to show Sales series
abline(v=c(2001,2002,2003,2004,2005,2006),lty=2)
plot(LSales,lwd=2) #to show log10 of Sales series
abline(v=c(2001,2002,2003,2004,2005,2006),lty=2)

# estimating the trend
par(mfrow=c(3,1))
ma12 <- c(1/24,rep(1/12,11),1/24)
print(sum(ma12^2)) # variance reduction rate 0.08
TrendSales<-filter(LSales,ma12,sides=2)
plot(LSales,type="l",lwd=2,main="log of Sales and Trend est.")
#lines(TrendSales,lty=3,lwd=3,col="red")
lines(TrendSales,type="p",pch=20,lwd=2,col="red")

SalesCVS0 <- LSales-TrendSales
C <- rep(0,12) ; Ccald <- C
for(i in 1:12){
C[i]=SalesCVS0[6+i]/5
for(k in 1:4){
C[i]=C[i]+SalesCVS0[6+i+k*12]/5}
}
C=C-sum(C)/12
Ccald[1]<-C[12]
for(i in 2:12) {Ccald[i] <- C[i-1]}

# seasonal adjusted series (CVS)
for (t in 1:72) {
  j <- floor((7+t-0.1)/12)
  i <- 7+t-12*j
  CVS[t] <- LSales[t]-Ccald[i]}
# plot(CVS,type="l")
CVSts <- ts(CVS,start=c(2000,8),frequency=12)
plot(LSales,type="l",lwd=2,main="log of Sales and seasonal adjusted series")
lines(CVSts,type="p",pch=20,lwd=2,col="red")


# calculate the fit series
fit <- rep(0,72) ; CVS <- rep(0,72)
fit[1:6] <- NA ; fit[67:72] <- NA
for (t in 7:66) {
j <- floor((t-5-0.1)/12)
i <- t-5-12*j
fit[t] <- TrendSales[t]+Ccald[i]}
fitts <- ts(fit,start=c(2000,8),frequency=12)
plot(LSales,type="l",lwd=2,main="log of Sales and fit series")
lines(fitts,type="p",pch=20,lwd=2,col="red")


######### Example 7.1

library(stats)
library(astsa)
library(TSA)
library(forecast) 
library(caschrono)
library(tseries)
library(urca)


Sunspot <- scan(file.choose(),skip=3)
#Sunspot <- scan(file="Sunspot_1700_2001.txt",skip=3)
Sunspot_ts<-ts(Sunspot,start=1700)
plot(Sunspot,type="l",lwd=2)
plot(Sunspot_ts,type="l",lwd=2)
lines(lowess(Sunspot_ts,f=.05),lwd=3,col=2)
lines(lowess(Sunspot_ts,f=.15),lwd=3,col=3)
lines(lowess(Sunspot_ts,f=.3),lwd=3,col=4)

lambda <- BoxCox.lambda(Sunspot) # lambda=0.3837
Sunspot_BC <- BoxCox(Sunspot,lambda)
Sunspot_sqrt <- sqrt(Sunspot)
par(mfrow=c(3,1))
plot(Sunspot,type="l")
plot(Sunspot_sqrt,type="l")
plot(Sunspot_BC,type="l")
qqnorm(Sunspot,ylim=c(-50,200),main="Sunspot rawdata")
qqline(Sunspot,col=2,lwd=2)
qqnorm(Sunspot_sqrt,ylim=c(-5,20),main="Square Root of Sunspot")
qqline(Sunspot_sqrt,col=2,lwd=2)
qqnorm(Sunspot_BC,ylim=c(-5,20),main="Box-Cox lambda=0.3837")
qqline(Sunspot_BC,col=2,lwd=2)

Sunspot_rootf<-ts(Sunspot_sqrt,frequency=11)
par(mfrow=c(2,1))
monthplot(Sunspot_rootf,main="monthplot:Sunspot_rootf")
seasonplot(Sunspot_rootf)


library(TSA)
perio <- periodogram(Sunspot_sqrt,main="Sunspot_sqrt spectral density")
#periodogram is a function of the TSA library
abline(h=0)
x0 <- perio$freq[which.max(perio$spec)] # x0=0.090625 , 1/x0 = 11.03448
x1 <- perio$freq[which.max(perio$spec[0:20])] # x1=0.009375 , 1/x1 = 106.67

par(mfrow=c(2,1))
plot(Sunspot_sqrt,type="l",main="Sunspot_sqrt",lwd=3)
lines(lowess(Sunspot_sqrt,f=.05),lwd=3,col=2)
lines(lowess(Sunspot_sqrt,f=.15),lwd=3,col=3)
lines(lowess(Sunspot_sqrt,f=.30),lwd=3,col=4)
diff_Sunspot_sqrt <- diff(Sunspot_sqrt)
plot(diff_Sunspot_sqrt,type="l",main="Differenced Sunspot_sqrt",lwd=3)
lines(lowess(diff_Sunspot_sqrt,f=.05),lwd=3,col=2)
lines(lowess(diff_Sunspot_sqrt,f=.15),lwd=3,col=3)
lines(lowess(diff_Sunspot_sqrt,f=.30),lwd=3,col=4)



# Trying to fit an arima(1,1,1) model
model_111 <- arima(Sunspot_sqrt,order=c(1,1,1))
summary(model_111)
t_stat(model_111)
tsdiag(model_111,gof.lag=70)
qqnorm(model_111$res) 
qqline(model_111$res,col=2,lwd=2)

Sun_sqrt_auto <- auto.arima(Sunspot_sqrt) ; Sun_sqrt_auto
Sun_auto_b_fix <- arima(Sunspot_sqrt,order=c(5,1,5),fixed=c(0,NA,0,NA,NA,NA,NA,NA,NA,NA))
Sun_auto_b_fix

Sun_modf <- Sun_auto_b_fix # shortername
Sun_fore <- predict(Sun_modf,16) # forecast sixteen years ahead
par(mfrow=c(2,1))
#plot(Sunspot_sqrt,xlim=c(1920,2025),ylim=c(0,15),lwd=2,ylab="Sunspot’s root and forecast")
plot(Sunspot_sqrt,xlim=c(220,325),lwd=2,type="l",ylab="Sunspot’s root and forecast")
up <- Sun_fore$pred+2*Sun_fore$se
low <- Sun_fore$pred-2*Sun_fore$se
xx=c(time(low),rev(time(up)))
yy=c(low,rev(up))
polygon(xx,yy,border=8,col=gray(.5,alpha=.3))
lines(Sun_fore$pred,type="p",col=2)
lines(Sun_fore$pred,type="l",col=2,lwd=2)
ts.plot(Sunspot_rootf,fitted(Sun_modf),col=c(1,2),ylim=c(0,15),lwd=2,ylab="Sunspot’s root and fitted")


######### Example 7.2

Man_empl <- scan(file.choose())
#Man_empl <- scan("male_empl_W9.txt")
par(mfrow=c(2,1))
Man_empl_ts <- ts(Man_empl,start=c(1971,1),frequency=12)
plot(Man_empl_ts,lwd=3)
Mtime <- time(Man_empl)
plot(Man_empl,type="l",lwd=3)
lines(lowess(Mtime,Man_empl,0.05),col=2,lwd=4)#red
lines(lowess(Mtime,Man_empl,0.10),col=3,lwd=4)#green
lines(lowess(Mtime,Man_empl,0.20),col=4,lwd=4)#blue
par(mfrow=c(2,1))
monthplot(Man_empl_ts)
seasonplot(Man_empl_ts)

k=kernel("modified.daniell",c(3,3))
Man_spec <- spec.pgram(Man_empl_ts,k,taper=0,log="no")
abline(v=c(1:5),lty="dotted")


adf.test(Man_empl,k=10)#p-value=0.69wedon’treject"unitroot"
kpss.test(Man_empl,null="Level") # p-value < 0.01 : we reject stationarity
kpss.test(Man_empl,null="Trend") # p-value < 0.01 : we reject stationarity

Man_arma12 <- arima(Man_empl,order=c(1,0,2),seasonal=list(order=c(0,0,0),period=12))
summary(Man_arma12)
# Attention must be given to the following pitfall:
# what R calls "intercept" is in fact the mean of the series
t_stat(Man_arma12)
tsdiag(Man_arma12,gof.lag=35)
Man_arma12b <- sarima(Man_empl,1,0,2,0,0,0,12)

Man_ma01_01 <- arima(Man_empl,order=c(0,1,1),seasonal=list(order=c(0,1,1),period=12))
summary(Man_ma01_01)
tsdiag(Man_ma01_01,gof.lag=30)
sarima(Man_empl,0,1,1,0,1,1,12)
t_stat(Man_ma01_01)

Man_final <- Man_ma01_01 # more easy name
Man_fore <- predict(Man_final,12) # twelve months ahead forecast
par(mfrow=c(2,1))
plot(Man_empl,col="black",type="l",
lwd=2,ylab="Man_empl and fitted")
lines(fitted(Man_final)[,1],lwd=3,col="red")
points(Man_empl,lwd=3,col="black")
plot(Man_final,xlim=c(60,145),lwd=2,ylab="Man_empl and forecast")
up <- Man_fore$pred+2*Man_fore$se
low <- Man_fore$pred-2*Man_fore$se
xx=c(time(low),rev(time(up)))
yy=c(low,rev(up))
polygon(xx,yy,border=8,col=gray(.5,alpha=.3))
lines(Man_fore$pred,type="p",col=2)
lines(Man_fore$pred,type="l",col=2,lwd=2)

library(TSA)
Man_outlier <- arimax(Man_empl,order=c(0,1,1),seasonal=list(order=c(0,1,1),period=12),xreg=(Mai80=1*(seq(Man_empl)==113)),method='ML')
innov_out_scale <- scale(resid(Man_outlier))
par(mfrow=c(1,2))
hist(innov_out_scale[-(1:12)],breaks=12,freq=FALSE,ylim=c(0,0.5))
curve(dnorm(x),add=TRUE,col="red",lwd=2)
qqnorm(innov_out_scale[-(1:12)])
qqline(innov_out_scale[-(1:12)],col=2)






