######### Example 1

par(mfrow=c(2,1))
set.seed(237)
ysim <- arima.sim(n=200,list(ar=c(0.3,-0.4),ma=-0.4),sd=sqrt(2),n.star=50)
plot(ysim,type="l",main="Sample for an ARMA(2,1)")
yintg <- cumsum(ysim)
plot(yintg,type="l",main="Sample for the deduced ARIMA(2,1,1)")
acf2(yintg) # suggests a non stationary series

yintg.ur <- ur.df(yintg,lags=6,type="drift")
summary(yintg.ur)
yintg.ur <- ur.df(yintg,lags=2,type="drift")
summary(yintg.ur)


#########  Example 2

library(forecast)
library(arfima)

set.seed(8555)
sim <- arfima.sim(1000,model=list(phi=c(0.2,0.1),dfrac=0.3,theta=0.2))
acf(sim,lag=60)

fit <- arfima(sim)
tsdisplay(residuals(fit)[[1]])
library(caschrono)
Box.test.2((residuals(fit)[[1]]),nlag=c(20,30,40,50,100)) # Box.test.2 is from library ‘‘caschrono’’
AIC(fit) # AIC=37.70134
BIC(fit) # BIC=52.4246
mod1_sim<-arima(sim,order=c(2,1,2))
AIC(fit,mod1_sim) # 35.84


######### Example 3

library(artfima)
data("ogden") # Saint Lawrence data
SLaw <- ogden ; SLawts <- ts(SLaw,start=1860)
par(mfrow=c(2,1))
plot(SLawts,type="l",ylab="StLawrence annual flow")
acf(SLaw,ylab="ACF for StLawrence annual flow")
SLaw_fit<-arfima(SLaw)
tsdisplay(residuals(SLaw_fit)[[1]])
Box.test.2((residuals(SLaw_fit)[[1]]),nlag=c(5,10,15,20,25))
SLaw_fitval <- SLaw-residuals(SLaw_fit)[[1]]
plot(SLaw,type="l",lwd=2,main="arfima d_hat=0.470")
lines(SLaw_fitval,lty="longdash")
AIC(SLaw_fit) # AIC = -522.3786
BIC(SLaw_fit) # BIC = -530.1027

Box.test.2(residuals(SLaw_fit)[[1]],nlag=c(5,10,15,20,25))


######### Example 4

data(LakeHuron) ; n <- length(LakeHuron) # n=98
t <- c(1:n) ; y <- LakeHuron
plot(y,type="l")
mod0.lake <- lm(y~t)
summary(mod0.lake)
Box.test(residuals(mod0.lake),lag=20,type="Ljung-Box",fitdf=2)
acf2(residuals(mod0.lake)) # suggests anar2model
library(nlme)
mod1.Lake <- gls(y~t,correlation=corARMA(form=~t,p=2,q=0),method="ML")
summary(mod1.Lake)

######### Example 5

mod2.Lake <- sarima(LakeHuron,2,0,0,xreg=t)
mod2.Lake$fit

#########  Example 6

data(bluebird)
lsales=bluebird[,1] ; price=bluebird[,2]
ps.dif=ts.intersect(diff(price),diff(lsales))
prewhiten(x=as.vector(ps.dif[,1]),y=as.vector(ps.dif[,2]),ylab="CCF")

blueb.m1=lm(lsales~price,data=bluebird)
summary(blueb.m1)
acf(residuals(blueb.m1),ci.type="ma")
pacf(residuals(blueb.m1))
#.....several trials for finding a convenient structure
# final fitted model below
blueb.m=arima(lsales,order=c(0,0,4),xreg=data.frame(price),fixed=c(0,NA,0,NA,NA,NA))
blueb.m1=sarima(lsales,0,0,4,xreg=data.frame(price),fixed=c(0,NA,0,NA,NA,NA))


######### Example 7

ar.repr <- ar(soi,AIC=TRUE)
ar.vec <- as.vector(ar.repr$ar)
ar.vec1 <- c(1,-ar.vec)
soi.pwG <- resid(ar.repr)
acf2(soi.pwG) # looks like WN
Box.test(soi.pwG,lag=40,fitdf=15)

rec.filG <- filter(rec,filter=ar.vec1,sides=1)
par(mfrow=c(2,1))
ccf(soi.pwG,rec.filG,xlim=c(-2,2),na.action=na.omit,ylab="CCFdirect")
prewhiten(y=rec,x=soi,ylab="CCF('prewhiten')")


fish=ts.intersect(rec,lag(rec,-1),lag(soi,-5))
RecL0 <- fish[,1];RecL1<-fish[,2];SoiL5<-fish[,3]
mod=lm(RecL0~RecL1+SoiL5,na.action=NULL)
u <- resid(mod)
acf2(u) # suggests ar 1
arx=sarima(RecL0,1,0,0,xreg=data.frame(RecL1,SoiL5))
# below: 1-step-ahead predictions
pred=rec-resid(arx$fit)
ts.plot(pred,rec,col=c("gray90",1),lwd=c(7,1))


######### Example 8

# Part 1.
library(TSA)
data(airmiles)
par(mfrow=c(1,1))
plot(log(airmiles),ylab="Log(airmiles)",xlab="Year",main="Log of monthly airline passenger-miles in US")
#
# Part 2.
#Looking at the differenced series before 2001-09
diff_air_unp <- as.vector(diff(diff(window(log(airmiles),end=c(2001,8)),12)))
par(mfrow=c(2,1))
acf(diff_air_unp,lag.max=36,main="ACF of the differenced log(airmiles) before 2001-09")
pacf(diff_air_unp,lag.max=36,main="PACF of the differenced log(airmiles) before 2001-09")
#
# Part 3.
diff_air_mod0 <- arimax(diff_air_unp,order=c(1,1,0),seasonal=list(order=c(0,1,0),period=12))
tsdiag(diff_air_mod0)
diff_air_mod1 <- arimax(diff_air_unp,order=c(0,1,1),seasonal=list(order=c(0,1,0),period=12))
tsdiag(diff_air_mod1)
#
# Part 4. A first fit with intervention
air.m0=arimax(log(airmiles),order=c(0,1,1),seasonal=list(order=c(0,1,0),period=12),xtransf=data.frame(I911=1*(seq(airmiles)==69),I911=1*(seq(airmiles)==69)),transfer=list(c(0,0),c(1,0)),method="ML")
summary(air.m0)
t_stat(air.m0)
tsdiag(air.m0)
#
# Part 5. fit with intervention and outliers, plot fitted values
air.m1=arimax(log(airmiles),order=c(0,1,1),seasonal=list(order=c(0,1,1),period=12),xtransf=data.frame(I911=1*(seq(airmiles)==69),I911=1*(seq(airmiles)==69)),transfer=list(c(0,0),c(1,0)),xreg=data.frame(Dec96=1*(seq(airmiles)==12),Jan97=1*(seq(airmiles)==13),Dec02=1*(seq(airmiles)==84)),method="ML")
summary(air.m1)
t_stat(air.m1)
tsdiag(air.m1)
par(mfrow=c(1,1))
plot(log(airmiles),ylab="Log(airmiles)",lwd=2)
points(fitted(air.m1))


#########  Example 9

library(vars)
y=cbind(cmort,tempr)
fit=VAR(y,p=1,type="both")
summary(fit)

par(mfrow=c(2,2))
stats::acf(resid(fit),52)
serial.test(fit,lags.pt=52,type="PT.asymptotic")

VARselect(x,lag.max=10,type="both")


