######### Section 4.1

library(astsa)

set.seed(101)
y <- arima.sim(n=200,list(ar=0.5,ma=-0.8))
mod1 <- arima(y,order=c(1,0,1),include.mean=FALSE)

num=length(y)
theta1=0 # just to define the form of the model
A <- array(c(theta1,1),dim=c(1,2,num)) # idem
phi1=0 # idem
Phi <- matrix(c(0,0,1,phi1),2,2) # idem
cR=0 # observation noise sigma_V=0
stdW=0 # just to define the model
cQ=matrix(c(0,0,0,stdW),2) # idem
mu0=c(1,1);Sigma0=matrix(c(1,0.5,0.5,1),2)
# Function to Calculate Likelihood
Linn=function(para){
stdW=para[1]
cQ=matrix(c(0,0,0,stdW),2)
cR=0 # sigma_V
phi1=para[2]
Phi=matrix(c(0,0,1,phi1),2) # put the matrix together
theta1=para[3]
A=array(c(theta1,1),dim=c(1,2,num))
kf=Kfilter1(num,y,A,mu0,Sigma0,Phi,0,0,cQ,cR,input=0)
return(kf$like)
}
# Estimation
init.par=c(1.2,0.4,0.6) # initial values of parameters
est=optim(init.par,Linn,NULL,method="Nelder-Mead",hessian=TRUE)
SE=sqrt(diag(solve(est$hessian)))


num=length(y)
theta1=0 # just to define the form of the model
A<-array(1,dim=c(1,1,num)) # idem
phi1=0 # idem
Phi<-phi1 # idem
cR=0 # observation noise sigma_V=0
stdW=0 # just to define the model
cQ=stdW # idem
mu0=0 ; Sigma0=1
# Function to Calculate the Likelihood
Linn=function(para){
stdW=para[1]
cQ=stdW
cR=cQ # sigma_V
S=stdW^2
phi1=para[2]
Phi=phi1
theta1=para[3]
Theta=phi1+theta1
A=array(1,dim=c(1,2,num))
kf=Kfilter2(num,y,A,mu0,Sigma0,Phi,0,0,Theta,cQ,cR,S,input=0)
return(kf$like)
}

######### Section 4.2

# Setup
y=Nile ; num=length(y)
input=as.matrix(rep(1,num))
A=array(1,dim=c(1,1,num));Phi=1
mu0=0;Sigma0=10000
# Function to Calculate Likelihood
Linn=function(para){
cQ=para[1] # sigma_W
cR=para[2] # sigma_V
const_y=para[3] # constant in observation equation
kf=Kfilter1(num,y,A,mu0,Sigma0,Phi,0,const_y,cQ,cR,input)
return(kf$like)
}
# Estimation
init.par=c(100,100,900) # initial values of parameters
est=optim(init.par,Linn,NULL,method="BFGS",hessian=TRUE)
SE=sqrt(diag(solve(est$hessian)))
# using method="Nelder-Mead" leads to slight different results.

# Summary of estimation
estimate=est$par;u=cbind(estimate,SE);u
# Smooth(first set parameters to their final estimates)
cQ=est$par[1]
cR=est$par[2]
const_y=est$par[3]
ks=Ksmooth1(num,y,A,mu0,Sigma0,Phi,0,const_y,cQ,cR,input)

# Plot of Kalman smoothed values of Nile flow
# with 95% confidence intervals
par(mfrow=c(1,1))
xsm=ts(as.vector(ks$xs),start=1871)+const_y
rmse=ts(sqrt(as.vector(ks$Ps)),start=1871)
plot(xsm,ylim=c(500,1500),main="Kalman smoothing",ylab="Nileflow")
lines(xsm)
xx=c(time(xsm),rev(time(xsm)))
yy=c(xsm-2*rmse,rev(xsm+2*rmse))
polygon(xx,yy,col=gray(.6,alpha=.25))
lines(y,type="o",pch=19,col=4,lty=6)

### Nile data. Plots for comparison of Kalman prediction, filtering and smoothing
par(mfrow=c(3,1))
# Prediction
xp=ts(as.vector(ks$xp),start=1871)+const_y
rmse=ts(sqrt(as.vector(ks$Pp)),start=1871)
plot(xp,ylim=c(500,1500),main="Prediction",ylab="Nileflow")
xx=c(time(xp),rev(time(xp)))
yy=c(xp-1.645*rmse,rev(xp+1.645*rmse))
polygon(xx,yy,col=gray(.6,alpha=.25))
lines(y,type="o",pch=19,col=4,lty=6)
# filtering
xf=ts(as.vector(ks$xf),start=1871)+const_y
rmse=ts(sqrt(as.vector(ks$Pf)),start=1871)
plot(xf,ylim=c(500,1500),main="Filtering",ylab="Nileflow")
xx=c(time(xf),rev(time(xf)))
yy=c(xf-1.645*rmse,rev(xf+1.645*rmse))
polygon(xx,yy,col=gray(.6,alpha=.25))
lines(y,type="o",pch=19,col=4,lty=6)
# smoothing
xsm=ts(as.vector(ks$xs),start=1871)+const_y
rmse=ts(sqrt(as.vector(ks$Ps)),start=1871)
plot(xsm,ylim=c(500,1500),main="Smoothing",ylab="Nileflow")
xx=c(time(xsm),rev(time(xsm)))
yy=c(xsm-1.645*rmse,rev(xsm+1.645*rmse))
polygon(xx,yy,col=gray(.6,alpha=.25))
lines(y,type="o",pch=19,col=4,lty=6)


#########  Example 7

# Nile with missing observation
y <- Nile-mean(Nile) ; num=length(y)
set.seed(100)
#unifsample <- runif(100)
unifsample <- runif(50)
ym <- y ; tresh <- 0.7
ym[51:100] <- y[51:100]*(unifsample>tresh)

# building the tilde series and its state space representation
ywt <- ym
A=array(0,dim=c(1,1,num))
for(k in 1:num) {if(ywt[k]!=0) A[,,k]=1}
# initialization
mu0=0 ; Sigma0=10000
Phi=1
cQ=100
cR=100
Linn=function(para){
cQ=para[1] # sigma_W
cR=para[2] # sigma_V
kf=Kfilter1(num,ywt,A,mu0,Sigma0,Phi,0,0,cQ,cR,input=0)
return(kf$like)
}

# Estimation
################### BEWARE THIS PART WILL NOT REPRODUCE THE FIGURE 4, PAGE 207, OF THE BOOK #######
init.par=c(100,100) # initial values of parameters
# est=optim(init.par,Linn,NULL,method="Nelder-Mead",hessian=TRUE) # yields an error
est=optim(init.par,Linn,NULL,method="BFGS",hessian=TRUE)
SE=sqrt(diag(solve(est$hessian)))
cQ=est$par[1] ; cR=est$par[2]
ks=Ksmooth1(num,ywt,A,mu0,Sigma0,Phi,0,0,cQ,cR,input=0)
ys=ks$xs[1,1,]+mean(Nile)
ys <- ts(as.vector(ys),start=1871)
p1=2*sqrt(ks$Ps[1,1,])

par(mfrow=c(1,1))
#plot(ys,type="p",col="red",ylim=c(100,1400),xlab="day")
plot(y+mean(Nile),type="p",col="red",ylim=c(-100,1400))
ymna <- ym ; ymna[which(ym==0)] <- NA
points(ymna+mean(Nile),pch=19) # missing values
lines(ys+p1,lty=2,col=4) ; lines(ys-p1,lty=2,col=4)
########################################################




#########  Example 8

fit <- StructTS(JohnsonJohnson,type="BSM")
fitall <- apply(fitted(fit),1,sum)
fit_ts <- ts(fitall,start=1960,frequency=4)
res <- JohnsonJohnson-fitall
sum_sq_res<-sum(res^2) # 3.308155
par(mfrow=c(1,1))
plot(JohnsonJohnson,lwd=2)
lines(fit_ts,col="red",lwd=2)
par(mfrow=c(4,1))
plot(cbind(fitted(fit),resids=resid(fit)),main="Johnson and Johnson")
